---@type Tinkr
local Tinkr = ...

---@param path string
---@param ...? any
local function require(path, ...)
    local func = nil
    local newPath = path
    local loader = nil
    if newPath:sub(-4) ~= ".lua" and newPath:sub(-5) ~= ".luac" then
        if FileExists(newPath .. ".lua") then
            newPath = newPath .. ".lua"
        elseif FileExists(newPath .. ".luac") then
            newPath = newPath .. ".luac"
        else
            Tinkr:OnError(string.format("File not found: path(%s) newPath(%s)", path, newPath), -1)
            return
        end
    end

    if newPath:sub(-4) == ".lua" then
        func = loadstringsecure(ReadFile(newPath) or "", newPath)
    elseif newPath:sub(-5) == ".luac" then
        func = RunEncrypted(newPath)
    else
        Tinkr:OnError(string.format("Loader not found: path(%s) newPath(%s)", path, newPath), -1)
        return
    end
    if type(func) == "string" then
        if type(func) == "string" then
            Tinkr:OnError(string.format("Loader did not return function: path(%s) newPath(%s) error(%s)", path, newPath, func or ""), -1)
            return
        end
    end
    setfenv(func, setmetatable({ ["_T"] = _T }, {
        __index = function(t, v)
            if v == "require" then
                --print(v, t.__file)
                return function(...)
                    return require(...)
                end
            end
            if v == '__file' then
                --print(v, newPath)
                return path
            end
            if v == 'TINKR_SECURE' then
                --print(v, t.__file, issecure(), Tinkr:GetSecure())
                return Tinkr:GetSecure()
            end
            return Tinkr.__ENV(t, v)
        end
    }))

    local call = { pcall(func, Tinkr, ...) }
    -- print('call', call[1], call[2])
    if not call[1] then
        -- print(self:OnLoadingScreen())
        Tinkr:OnError(call[2], -1)
    end
    table.remove(call, 1)

    return unpack(call)
end
require("scripts/bastion/src/Bastion/Bastion"):Load()
